open GMain
  
type formule = 
    SINGLE of string
  | IMPLIC of formule * formule
  | AND of formule * formule
  | OR of formule * formule
  | FALSE;;


type sequent = {hyp : formule list;
		f : formule };;


exception RegleNonApplicable;;
exception Erreur_syntaxe;;
exception FormulePasIndiquee ;;

(* ---------------------ANALYSE SYNTAXIQUE--------------------- *)


(* retourne la chane de caractres s sans les espaces *)
let rec elimine_espace s =
  match s with
    "" -> ""
  | _ -> 
      if s.[0] = ' ' then 
	elimine_espace (String.sub s 1 ((String.length s) - 1))
      else 
	(String.make 1 s.[0])^
	(elimine_espace (String.sub s 1 ((String.length s) - 1)));;

let i = ref (-1);;

(* retourne l'indice de la parenthse qui ferme la parenthse ouvrante de s *) 
let rec find1 s  = 
  if ((String.sub s 0 1)= "(") then (
    i:=!i+1;  
    find1 (String.sub s 1 ((String.length s)-1))
   )
  else (
    if ((String.sub s 0 1)= ")") then 
      ( 
	if ( !i = 0 ) then (String.length s)
	else  (
	  i :=!i-1;
	  find1 (String.sub s 1 ((String.length s)-1))
	 )
       )
    else ( 
      find1 (String.sub s 1 ((String.length s)-1)))
   )
      
(* transforme une chaine de caractres vers une formule *)        
let rec transforme s0 = 
  try(
    let s = elimine_espace s0 in
    match (String.sub s 0 1) with "A"|"B"|"C"|"D"|"E"|"F"|"G"|"H"|"I"|
      "J"|"K"|"L"|"M"|"N"|"O"|"P"|"Q"|"R"|
      "S"|"T"|"U"|"V"|"W"|"X"|"Y"|"Z" ->
        (
	 if (String.length s ==1) then 
           SINGLE (String.sub s 0 1)
         else 
           ( 
	     match (String.sub s 1 2) 
             with "->" ->
               IMPLIC (SINGLE (String.sub s 0 1) ,
		       transforme (String.sub s 3 ((String.length s)-3))) 
             | "an" ->
		 AND (SINGLE (String.sub s 0 1) ,
		      transforme (String.sub s 4 ((String.length s)-4)))
             | "or" ->
		 OR  (SINGLE (String.sub s 0 1) ,
		      transforme (String.sub s 3 ((String.length s)-3)))
             | _ -> raise Erreur_syntaxe 
	    )
	)
    |"(" -> 
	(
	 i := -1;
	 let ind =((String.length s)-(find1  s))  in 
         if (ind = ((String.length s)-1)) then 
	   (
	    transforme (String.sub s 1 (ind-1))
	   )
	 else(
	   match (String.sub s (ind+1) 2) with 
	     "->" -> 
	       IMPLIC(transforme(String.sub s 1 (ind-1)),
		      transforme(String.sub s (ind+3)((String.length s)-ind-3))) 
           | "an" -> 
	       AND (transforme (String.sub s 1 (ind-1)),
                   transforme (String.sub s (ind+4)((String.length s)-ind-4))) 
           | "or" ->
	       OR  (transforme (String.sub s 1 (ind-1)),
                    transforme (String.sub s (ind+3)((String.length s)-ind-3)))
           | _ -> raise Erreur_syntaxe
          )
	) 
    | _ -> 
	if ((String.sub s 0 5)="false") then 
	  FALSE 
	else  
	  raise Erreur_syntaxe
   )
  with Invalid_argument "String.sub"  -> raise Erreur_syntaxe;;

(* retourne l'indice de la position de la caractre s1 dans s *)
let rec find2 s s1 = 
  try
    (
     if ( (s <> "") & ((String.sub s ((String.length s)-1) 1 )= s1)) then 
       ((String.length s)-1) 
     else
       (
	if ((String.length s)=1) then -1
	else (
	  (find2 (String.sub s 0 ((String.length s)-1)) s1)
	 )
       )
    )with Invalid_argument "String.sub"
    -> raise Erreur_syntaxe;;

(* retourne une liste de chaine de caractres en cherchant les ',' *)
let rec trans_to_list s0 = 
  let s = elimine_espace s0 in
  if (s = "") then [] else
  (
   let ind = -1 in 
   let ind = find2 s "," in
   if (ind = -1 ) then [s]
   else (
     ((trans_to_list (String.sub s 0 (ind)))@
      [(String.sub s (ind+1) ((String.length s)- ind-1))])
    )
  );;

(* retourne une liste de formule  partir d'une chaine de caractres *)
let transforme2 s =
  List.map transforme (trans_to_list s);;  


(* transforme une formule en une chaine de caractres *)
let rec trans_to_string f = 
  match f with 
    SINGLE s -> s
  |IMPLIC (f1,f2) -> 
      ("( "^(trans_to_string f1)^" -> "^(trans_to_string f2)^" )")
  |AND (f1,f2) -> 
      ("( "^(trans_to_string f1)^" and "^(trans_to_string f2)^" )")
  |OR (f1,f2) -> 
      ("( "^(trans_to_string f1)^" or "^(trans_to_string f2)^" )")
  | FALSE -> "false";;


(* transforme une liste de formule en une chaine de caractres *) 
let trans_to_string2 l1 =
  let l2  =List.map trans_to_string l1 in
  List.fold_left 
    (fun x -> fun y ->if (x <> "") then (x^" , "^y) else (y) ) "" l2
;;


(* ---------------------REGLES DE DEDUCTION--------------------- *)


(* renvoie true si a est un lment de la liste f et false sinon *)
let rec find f a =
  match f with
    [] -> false
  | t::q -> (if (t=a) then true
  else find q a);;


(* rgle d'hypothse *)
let hypo s =
  if (find s.hyp s.f) then []
  else raise RegleNonApplicable;;



(* rgle d'introduction du => *)
let introImplic s =
  match s.f with
    IMPLIC (a,b) -> 
      if not(find s.hyp a) then 
	[{hyp = a::s.hyp ; f = b}] 
      else [{hyp = s.hyp ; f = b}]
  | _ -> raise RegleNonApplicable;;


(* rgle d'limination du => *)
let elimImplic s a =
  [{hyp = s.hyp ; f = IMPLIC (a, s.f)} ;
   {hyp = s.hyp ; f = a}];;


(* rgle d'introduction du /\ *)
let introEt s =
  match s.f with
    AND (a,b) -> [{hyp = s.hyp ; f = a} ;
		  {hyp = s.hyp ; f = b}]
  | _ -> raise RegleNonApplicable;;


(* rgle d'limination left du /\ *)
let elimLeftEt s b =
  [{hyp = s.hyp ; f = AND (s.f, b)}];;


(* rgle d'limination right du /\ *)
let elimRightEt s a =
  [{hyp = s.hyp ; f = AND (a, s.f)}];;


(* rgle d'introduction left du \/ *)
let introLeftOu s =
  match s.f with
    OR (a,b) -> [{hyp = s.hyp ; f = a}]
  | _ -> raise RegleNonApplicable;;


(* rgle d'introduction right du \/ *)
let introRightOu s =
  match s.f with
    OR (a,b) -> [{hyp = s.hyp ; f = b}]
  | _ -> raise RegleNonApplicable;;


(* rgle d'limination du \/ *)
let elimOu s a b =
  if not(find s.hyp a) then 
    if not(find s.hyp b) then 
      [{hyp = s.hyp ; f = OR (a, b)} ;
       {hyp = a::s.hyp ; f = s.f} ;
       {hyp = b::s.hyp ; f = s.f}]
    else
      [{hyp = s.hyp ; f = OR (a, b)} ;
       {hyp = a::s.hyp ; f = s.f} ;
       {hyp = s.hyp ; f = s.f}]
  else
    [{hyp = s.hyp ; f = OR (a, b)} ;
     {hyp = s.hyp ; f = s.f} ;
     {hyp = s.hyp ; f = s.f}];;

(* rgle d'limination du FALSE *)
let elimNot s =
  [{hyp = s.hyp ; f = FALSE}];;

(* -------------------------INTERFACE-------------------------- *)


(*la liste contenant les sequents sur lesquelles on peut appliquer une rgle *)
let list_txt =ref [] ;;

(* supprime les lment contenus dans une "box" *)
let rec vider b = 
  let l= b#children in
  for i=0 to ((List.length l)-1) do (
    b#remove (List.nth l i);
   )done;;


let main () =
  (* la fenetre principale *)
  let window = GWindow.dialog ~title:"Assistant de preuve" ~width:900 
      ~height:750
      ~border_width:0 () in
  window#connect#destroy ~callback:Main.quit;
  
  let scrolled_window = GBin.scrolled_window ~border_width:10
      ~hpolicy:`AUTOMATIC ~vpolicy:`AUTOMATIC ~packing:window#vbox#add ()  in  
  let box = GPack.hbox ~packing:scrolled_window#add_with_viewport ()  in
  let box0 = GPack.vbox ~spacing:10 ~border_width:50
      ~packing:box#pack () in
  let b0 = GButton.button ~label:"Recommencer"
      ~packing:box0#pack () in
  let b1 = GButton.button ~label:"Retour" 
      ~packing:box0#pack () in
  GMisc.label ~packing:box0#add  ~line_wrap:true () ;
  let frame = GBin.frame ~label:"Les Regles de deductions"
      ~packing:box0#pack () in
  let box_err = GPack.hbox ~packing:box0#pack () in
  let erreur = GMisc.label ~packing:((box_err)#pack ~from:`END) 
      ~line_wrap:true () in
  GMisc.label ~packing:box0#add  ~line_wrap:true () ;
  
  let button0 = GButton.radio_button ~label:"Choisissez une regle"
      ~active:true ~packing:box0#add () in
  let button1 = GButton.radio_button
      ~group:button0#group ~label:"Hypothese" ~packing:box0#add () in                                            
  let button2 = GButton.radio_button 
      ~group:button0#group ~label:"I Impl" ~packing:box0#add () in
  let button3 = GButton.radio_button
      ~group:button0#group ~label:"I and" ~packing:box0#add () in
  let button4 = GButton.radio_button
      ~group:button0#group ~label:"I left or" ~packing:box0#add () in
  let button5 = GButton.radio_button
      ~group:button0#group ~label:"I right or" ~packing:box0#add () in
  let button6 = GButton.radio_button
      ~group:button0#group ~label:"E FALSE" ~packing:box0#add () in 
  let separator = GMisc.separator `HORIZONTAL ~packing: box0#pack () in
  let button7 = GButton.radio_button
      ~group:button0#group ~label:"E Impl"  ~packing:box0#add () in
  let button8 = GButton.radio_button
      ~group:button0#group ~label:"E left and" ~packing:box0#add () in
  let button9 = GButton.radio_button
      ~group:button0#group ~label:"E right and" ~packing:box0#add () in
  let button10 = GButton.radio_button
      ~group:button0#group ~label:"E or" ~packing:box0#add () in
  GMisc.label ~text:" Formule(s) a indiquer "
    ~packing:box0#add ~line_wrap:true ();
  let entry0 = GEdit.entry ~text:"  " ~max_length:1000 ~width:50
      ~packing:box0#add () in
  let entry01 = GEdit.entry ~text:"  " ~max_length:1000 ~width:50
      ~packing:box0#add () in
  let box1 = GPack.vbox ~packing:box#pack () in
  let box01 =GPack.hbox ~spacing:10 ~border_width:10
      ~packing:(box1#pack ~from:`END) () in
  let entry1 = GEdit.entry ~text:"  " ~max_length:500 
      ~packing:box01#add () in
  GMisc.label ~text:" |- " ~packing:box01#add ~line_wrap:true ();
  let entry2 = GEdit.entry ~text:"  " ~max_length:1000 ~width:400
      ~packing:box01#add () in
  let separator = GMisc.separator `HORIZONTAL 
      ~packing:(box1#pack ~from:`END)  () in
  let branche_active =ref [box1] in
  let branche_visitee = ref [box1] in
  let txt_visite = ref [] in
  let dern_regle = ref [] in
  
(* la fonction concernant le bouton de la rgle Hyp *)
  button1#connect#toggled  ~callback:(fun _->
    if (button1#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
           if (a1 <> "  ")then (
	     let f1 = transforme2 a1 in
	     let f2 = transforme a2 in
	     let s1 = { hyp = f1;f = f2} in
	     let s = hypo s1 in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(HYP)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"branche prouvee" 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     list_txt := [];
	     dern_regle := [] ;
	     erreur#set_text "****PREUVE TERMINEE****";                                     branche_active := List.tl (!branche_active);
	     for i=1 to 5000000 do()done ;
	     button0#set_active true
	    )else
	     (
	      entry1#set_editable true;
	      entry2#set_editable true;
	      if (a2 <> "  ") then raise RegleNonApplicable
	      else
		(
		 erreur#set_text "entrez un sequant";
		 button0#set_active true;
		)
	     )
	  )
	 else 
	   (
	    let s1 = (List.hd !list_txt)in
	    let s = hypo s1 in
	    let box = List.hd (!branche_active) in
	    erreur#set_text "  ";
	    (GMisc.label  ~text:"(HYP)" ~xalign:0.0 ~yalign:0.0 
	       ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    (GMisc.label  ~text:"branche prouvee" 
	       ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    (GMisc.label  ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	    list_txt := List.tl (!list_txt);
	    dern_regle := ["hyp"]@(!dern_regle) ;
	    if ((!list_txt) = [])then 
	      (
	       erreur#set_text "****PREUVE TERMINEE****";
	       dern_regle :=[];
	      );
            if (box <> (List.hd (!branche_visitee))) then  
	      branche_visitee := [box]@(!branche_visitee);
	    branche_active := List.tl (!branche_active);
	    for i=1 to 5000000 do()done ;
	    button0#set_active true
	   )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true)
       | Erreur_syntaxe ->  (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )  
      )
				     ) ;    
  
  
(* la fonction concernant le bouton de la rgle introImplic *)  
  button2#connect#toggled  ~callback:(fun _->
    if (button2#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
           if (a1 <> "  ")then (
	     let f1 = transforme2 a1 in
	     let f2 = transforme a2 in
	     let s1 = { hyp = f1;f = f2} in
	     let s2 = List.hd ( introImplic s1) in 
	     let t = trans_to_string2 s2.hyp in
	     let txt =(t^" |- "^(trans_to_string s2.f)) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(I IMPL)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     list_txt := [s2];
	     dern_regle := ["I Impl"] ;
	     for i=1 to 5000000 do()done ;
             button0#set_active true
	    )else
	     (
	      if (a2 <> "  ") then(
		let f2 = transforme a2 in
		let s1 = { hyp = [];f = f2} in
		let s2 = List.hd ( introImplic s1) in 
		let t = trans_to_string2 s2.hyp in
		let txt =(t^" |- "^(trans_to_string s2.f)) in
		let box = List.hd (!branche_active) in
		erreur#set_text "  ";
		(GMisc.label  ~text:"(I IMPL)" ~xalign:0.0 ~yalign:0.0 
		   ~packing:((box)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label ~text:txt 
		   ~packing:((box)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		let separator = GMisc.separator `HORIZONTAL 
		    ~packing: ((box)#pack ~from:`END) () in
		list_txt := [s2]@(!list_txt);
		dern_regle := ["I Impl"];)
	      else ( 
		entry1#set_editable true;
		entry2#set_editable true;
		erreur#set_text "entrez un sequent";
	       );
	      for i=1 to 5000000 do()done ;
	      button0#set_active true
	     )
	  )
	 else 
	   (
	    let s1 = (List.hd !list_txt)in
	    let s2 = List.hd ( introImplic s1) in 
	    let t = trans_to_string2 s2.hyp in
	    txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	    list_txt := List.tl (!list_txt) ;
	    list_txt := [s2]@(!list_txt) ;
	    let txt =(t^" |- "^(trans_to_string s2.f)) in
	    let box = List.hd (!branche_active) in
	    erreur#set_text "  ";
	    (GMisc.label  ~text:"(I IMPL)" ~xalign:0.0 ~yalign:0.0 
	       ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    if ((List.hd (!dern_regle)) = "E Impl") 
	    then  
	      (
	       (GMisc.label ~text:("V***********  "^txt) 
		  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       erreur#set_text "COUPURE";
	      )
	    else (GMisc.label ~text:txt 
		 ~packing:((box)#pack ~from:`END) 
		    ~line_wrap:true ())#set_use_underline true ;
	    let separator = GMisc.separator `HORIZONTAL 
		~packing: ((box)#pack ~from:`END) () in
	    dern_regle := ["I Impl"]@(!dern_regle);
	    for i=1 to 5000000 do()done ;
	    if (box <> (List.hd (!branche_visitee))) then  
	      branche_visitee := [box]@(!branche_visitee);
	    button0#set_active true
	   )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;

(* la fonction concernant le bouton de la rgle introEt *)  
  button3#connect#toggled  ~callback:(fun _->
    if (button3#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
	   print_string ("2"^a1^"2");
           if (a1 <> "  ")then (
	     let f1 = transforme2 a1 in
	     let f2 = transforme a2 in
	     let s0 = { hyp = f1;f = f2} in
	     let l = introEt s0 in
	     let s1 = List.hd l in 
	     let s2 = List.nth l 1 in
	     let t1 = trans_to_string2 s1.hyp in
	     let t2 = trans_to_string2 s2.hyp in
	     let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	     let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		 ~packing:((box)#pack ~from:`END) () in
	     let box03 = GPack.vbox ~packing:box02#pack () in
	     let box04 = GPack.vbox ~packing:box02#pack () in
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"(I AND)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
             let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     list_txt := [s1;s2];
	     dern_regle := ["I and"] ;
	     for i=1 to 5000000 do()done ;
	     branche_active := [box03]@[box04];
             button0#set_active true
	    )else
	     (
	      if (a2 <> "  ") then (
		let f2 = transforme a2 in
		let s0 = { hyp = [];f = f2} in
		let l = introEt s0 in
		let s1 = List.hd l in 
		let s2 = List.nth l 1 in
		let t1 = trans_to_string2 s1.hyp in
		let t2 = trans_to_string2 s2.hyp in
		let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
		let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
		let box = List.hd (!branche_active) in
		erreur#set_text "  ";
		let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		    ~packing:((box)#pack ~from:`END) () in
		let box03 = GPack.vbox ~packing:box02#pack() in
		let box04 = GPack.vbox ~packing:box02#pack () in
		(GMisc.label  ~packing:((box)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label  ~text:"(I AND)" ~xalign:0.0 ~yalign:0.0 
		   ~packing:((box03)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label  ~packing:((box04)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END)  
		   ~line_wrap:true ())#set_use_underline true ;
		(GMisc.label  ~packing:((box)#pack ~from:`END) 
		   ~line_wrap:true ())#set_use_underline true ;
		let separator = GMisc.separator `HORIZONTAL 
		    ~packing: ((box03)#pack ~from:`END) () in
		let separator = GMisc.separator `HORIZONTAL 
		    ~packing: ((box04)#pack ~from:`END) () in
		list_txt := [s1;s2];
		dern_regle := ["I and"] ;
		branche_active := [box03]@[box04];)
	      else
		(
		 entry1#set_editable true;
		 entry2#set_editable true;
		 erreur#set_text "entrez un sequent";
		);
	      for i=1 to 5000000 do()done ;
	      button0#set_active true
	     )
	  )
	 else (
	   let s0 = (List.hd !list_txt) in
	   let l = introEt s0 in
	   let s1 = List.hd l in 
	   let s2 = List.nth l 1 in
	   let t1 = trans_to_string2 s1.hyp in
	   let t2 = trans_to_string2 s2.hyp in
	   let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	   let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   let box02 =GPack.hbox ~homogeneous:true ~spacing:7
	       ~packing:((box)#pack ~from:`END) () in
	   let box03 = GPack.vbox ~packing:box02#pack () in
	   let box04 = GPack.vbox ~packing:box02#pack () in
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~text:"(I AND)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label  ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box03)#pack ~from:`END) () in
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box04)#pack ~from:`END) () in
           txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	   list_txt := List.tl (!list_txt);
	   list_txt := [s1;s2]@(!list_txt);
	   dern_regle := ["I and"]@(!dern_regle);
	   for i=1 to 5000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   branche_active := List.tl (!branche_active); 
	   branche_active := box03::box04::(!branche_active);
           button0#set_active true
	  )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle introLeftOu *)
  button4#connect#toggled  ~callback:(fun _->
    if (button4#active)then
      ( 
        try(
	  if (entry1#editable & entry2#editable) then (
	    let a1 = entry1#text in
	    let a2 = entry2#text in
	    entry1#set_editable false;
	    entry2#set_editable false;
	    if (a1 <> "  ")then (
	      let f1 = transforme2 a1 in
	      let f2 = transforme a2 in
	      let s1 = { hyp = f1;f = f2} in
	      let l = introLeftOu s1 in
	      let s2 = List.hd l in 
	      let t = trans_to_string2 s2.hyp in
	      let txt =(t^" |- "^(trans_to_string s2.f)) in
	      let box = List.hd (!branche_active) in
	      erreur#set_text "  " ;
	      (GMisc.label  ~text:"(I LEFT OR)" ~xalign:0.0 ~yalign:0.0 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      (GMisc.label ~text:txt 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      let separator = GMisc.separator `HORIZONTAL 
		  ~packing: ((box)#pack ~from:`END)() in
	      list_txt := [s2];
	      dern_regle := ["I left or"] ;
	      for i=1 to 5000000 do()done ;
              button0#set_active true
	     )else
	      (
	       if (a2 <>"  ") then 
		 (
		  let f2 = transforme a2 in
		  let s1 = { hyp = [];f = f2} in
		  let s2 = List.hd ( introLeftOu s1) in 
		  let t = trans_to_string2 s2.hyp in
		  let txt =(t^" |- "^(trans_to_string s2.f)) in
		  let box = List.hd (!branche_active) in
		  erreur#set_text "  ";
		  (GMisc.label  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt 
		  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box)#pack ~from:`END) () in
		  list_txt := [s2];
		  dern_regle :=  ["I left or"] ;
		 )
	       else
		 (
		  entry1#set_editable true;
		  entry2#set_editable true;
		  erreur#set_text "entrez un sequent";
		 );
	       for i=1 to 5000000 do()done ;
	       button0#set_active true
	      )
	   )
	  else 
	    (
	     let s1 = (List.hd !list_txt) in
	     let s2 = List.hd ( introLeftOu s1) in 
	     let t = trans_to_string2 s2.hyp in
	     txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	     list_txt := List.tl (!list_txt);
	     list_txt := [s2]@(!list_txt) ;
	     dern_regle :=  ["I left or"]@(!dern_regle) ;
	     let txt =(t^" |- "^(trans_to_string s2.f)) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     (GMisc.label  ~text:"(I LEFT OR)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     for i=1 to 5000000 do()done ;
	     if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     button0#set_active true
	    )
	 )with RegleNonApplicable -> (
	   erreur#set_text "RegleNonApplicable" ;
	   button0#set_active true
	  )
	| Erreur_syntaxe -> (
	    erreur#set_text "Erreur Syntaxique" ;
	    entry1#set_editable true;
	    entry2#set_editable true;
	    button0#set_active true
	   )
       )
				     ) ;
 
(* la fonction concernant le bouton de la rgle introRightOu *) 
  button5#connect#toggled  ~callback:(fun _->
    if (button5#active)then
      ( 
	try(
	  if (entry1#editable & entry2#editable) then (
	    let a1 = entry1#text in
	    let a2 = entry2#text in
	    entry1#set_editable false;
	    entry2#set_editable false;
	    if (a1 <> "  ")then (
	      let f1 = transforme2 a1 in
	      let f2 = transforme a2 in
	      let s1 = { hyp = f1;f = f2} in
	      let l = introRightOu s1 in
	      let s2 = List.hd l in 
	      let t = trans_to_string2 s2.hyp in
	      let txt =(t^" |- "^(trans_to_string s2.f)) in
	      let box = List.hd (!branche_active) in
	      (GMisc.label  ~text:"(I RIGHT OR)" ~xalign:0.0 ~yalign:0.0 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      (GMisc.label ~text:txt 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      let separator = GMisc.separator `HORIZONTAL 
		  ~packing: ((box)#pack ~from:`END) () in
	      list_txt := [s2];
	      dern_regle :=  ["I right or"] ;
	      for i=1 to 5000000 do()done ;
              button0#set_active true
	     )else
	      (
	       if (a2 <> "  ")then 
		 (
		  let f2 = transforme a2 in
		  let s1 = { hyp = [];f = f2} in
		  let s2 = List.hd ( introRightOu s1) in 
		  let t = trans_to_string2 s2.hyp in
		  let txt =(t^" |- "^(trans_to_string s2.f)) in
		  let box = List.hd (!branche_active) in
		  (GMisc.label  ~text:"(I RIGHT OR)" ~xalign:0.0 ~yalign:0.0 
		     ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt 
		   ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box)#pack ~from:`END) () in
		  list_txt := [s2];
		  dern_regle :=  ["I right or"] ;)else
		 (
		  entry1#set_editable true;
		  entry2#set_editable true;
		  erreur#set_text "entrez un sequent";
		 );
	       for i=1 to 5000000 do()done ;
	       button0#set_active true
	      )
	   )
	  else 
	    (
	     let s1 = (List.hd !list_txt) in
	     let s2 = List.hd ( introRightOu s1) in 
	     let t = trans_to_string2 s2.hyp in
	     txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	     list_txt := List.tl (!list_txt);
	     list_txt := [s2]@(!list_txt) ;
     	     dern_regle :=  ["I right or"]@(!dern_regle) ;
	     let txt =(t^" |- "^(trans_to_string s2.f)) in
	     let box = List.hd (!branche_active) in
	     (GMisc.label  ~text:"(I RIGHT OR)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt 
		~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box)#pack ~from:`END) () in
	     for i=1 to 5000000 do()done ;
	     if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     button0#set_active true
	    )
	 )with RegleNonApplicable -> (
	   erreur#set_text "RegleNonApplicable" ;
	   button0#set_active true
	  )
	| Erreur_syntaxe -> (
	    erreur#set_text "Erreur Syntaxique" ;
	    entry1#set_editable true;
	    entry2#set_editable true;
	    button0#set_active true
	   )
       )
				     );
  
(* la fonction concernant le bouton de la rgle elimNot *)  
  button6#connect#toggled  ~callback:(fun _->
    if (button6#active)then
      ( 
        try(if (entry1#editable & entry2#editable) then (
	  let a1 = entry1#text in
	  let a2 = entry2#text in
	  entry1#set_editable false;
	  entry2#set_editable false;
	  if (a1 <> "  ")then (
	    let f1 = transforme2 a1 in
	    let f2 = transforme a2 in
	    let s1 = { hyp = f1;f = f2} in
	    let l = elimNot s1 in
	    let s2 = List.hd l in 
	    let t = trans_to_string2 s2.hyp in
	    let txt =(t^" |- "^(trans_to_string s2.f)) in
	    let box = List.hd (!branche_active) in
	    erreur#set_text "  ";
	    (GMisc.label  ~text:"(E FALSE)" ~xalign:0.0 ~yalign:0.0  
	       ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    (GMisc.label ~text:txt 
	       ~packing:((box)#pack ~from:`END) 
	       ~line_wrap:true ())#set_use_underline true ;
	    let separator = GMisc.separator `HORIZONTAL 
		~packing: ((box)#pack ~from:`END) () in
	    list_txt := [s2];
	    dern_regle :=  ["E false"] ;
	    for i=1 to 5000000 do()done ;
            button0#set_active true
	   )else
	    (
	     if(a2<> "  ") then (
	       let f2 = transforme a2 in
	       let s1 = { hyp = [];f = f2} in
	       let s2 = List.hd ( elimNot s1) in 
	       let t = trans_to_string2 s2.hyp in
	       let txt =(t^" |- "^(trans_to_string s2.f)) in
	       let box = List.hd (!branche_active) in
	       erreur#set_text "  ";
	       (GMisc.label  ~text:"(E FALSE)" ~xalign:0.0 ~yalign:0.0 
		  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt 
		  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box)#pack ~from:`END) () in
	       list_txt := [s2];
	       dern_regle :=  ["E false"] ;
	      )
	     else
	       (
		entry1#set_editable true;
		entry2#set_editable true;
		erreur#set_text "entrez un sequent";
	       );
	     for i=1 to 5000000 do()done ;
	     button0#set_active true
	    )
	 )
	else 
	  (
	   let s1 = (List.hd !list_txt)in
	   let s2 = List.hd ( elimNot s1) in 
	   let t = trans_to_string2 s2.hyp in
	   txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	   list_txt := List.tl (!list_txt);
	   list_txt := [s2]@(!list_txt) ;
	   dern_regle := ["E false"]@(!dern_regle) ;
	   let txt =(t^" |- "^(trans_to_string s2.f)) in
	   let box = List.hd (!branche_active) in
	   erreur#set_text "  ";
	   (GMisc.label  ~text:"(E FALSE)" ~xalign:0.0 ~yalign:0.0 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   (GMisc.label ~text:txt 
	      ~packing:((box)#pack ~from:`END) 
	      ~line_wrap:true ())#set_use_underline true ;
	   let separator = GMisc.separator `HORIZONTAL 
	       ~packing: ((box)#pack ~from:`END) () in
	   for i=1 to 5000000 do()done ;
	   if (box <> (List.hd (!branche_visitee))) then  
	     branche_visitee := [box]@(!branche_visitee);
	   button0#set_active true
	  )
	   )with RegleNonApplicable -> (
	     erreur#set_text "RegleNonApplicable" ;
	     button0#set_active true
	    )
	| Erreur_syntaxe -> (
	    erreur#set_text "Erreur Syntaxique" ;
	    entry1#set_editable true;
	    entry2#set_editable true;
	    button0#set_active true
	   )
       )
				     ) ;

(* la fonction concernant le bouton de la rgle elimImplic *)  
  button7#connect#toggled  ~callback:(fun _->
    if (button7#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
	   if (a0 <> "  ")then (
             if (a1 <> "  ")then (
	       let f0 = transforme a0 in
	       let f1 = transforme2 a1 in
	       let f2 = transforme a2 in
	       let s0 = { hyp = f1;f = f2} in
	       let l = elimImplic s0  f0 in
	       let s1 = List.hd l in 
	       let s2 = List.nth l 1 in
	       let t1 = trans_to_string2 s1.hyp in
	       let t2 = trans_to_string2 s2.hyp in
	       let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	       let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	       let box = List.hd (!branche_active) in
	       erreur#set_text "  ";
	       let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		   ~packing:((box)#pack ~from:`END) () in
	       let box03 = GPack.vbox ~packing:box02#pack () in
	       let box04 = GPack.vbox ~packing:box02#pack () in
	       (GMisc.label  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~text:"(E IMPL)" ~xalign:0.0 ~yalign:0.0 
		  ~packing:((box03)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box03)#pack ~from:`END) () in
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box04)#pack ~from:`END) () in
	       list_txt := [s1;s2];
	       dern_regle := ["E Impl"];
	       for i=1 to 5000000 do()done ;
	       if (box <> (List.hd (!branche_visitee))) then  
		 branche_visitee := [box]@(!branche_visitee);
	       branche_active := [box03]@[box04];
               button0#set_active true;
	       entry0#set_text "  "
	      )else
	       (
		if (a2 <> "  ") then (
		  let f0 = transforme a0 in
		  let f2 = transforme a2 in
		  let s0 = { hyp = [];f = f2} in
		  let l = elimImplic s0  f0 in
		  let s1 = List.hd l in 
		  let s2 = List.nth l 1 in
		  let t1 = trans_to_string2 s1.hyp in
		  let t2 = trans_to_string2 s2.hyp in
		  let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
		  let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
		  let box = List.hd (!branche_active) in
		  erreur#set_text "  ";
		  let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		      ~packing:((box)#pack ~from:`END) () in
		  let box03 = GPack.vbox ~packing:box02#pack() in
		  let box04 = GPack.vbox ~packing:box02#pack () in
		  (GMisc.label  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~text:"(E IMPL)" ~xalign:0.0 ~yalign:0.0 
		     ~packing:((box03)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END)  
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box03)#pack ~from:`END) () in
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box04)#pack ~from:`END) () in
		  list_txt := [s1;s2];
		  dern_regle := ["E Impl"];
		  if (box <> (List.hd (!branche_visitee))) then  
		    branche_visitee := [box]@(!branche_visitee);
		  branche_active := [box03]@[box04];)
		else
		  (
		   entry1#set_editable true;
		   entry2#set_editable true;
		   erreur#set_text "entrez un sequent";
		  );
		for i=1 to 5000000 do()done ;
                button0#set_active true;
		entry0#set_text "  "
	       )
	    )
	   else (  
	     button0#set_active true;
	     raise FormulePasIndiquee 
	    )
	  )
	 else (
	   let a0 = entry0#text in
	   if (a0 <> "  ")then (
	     let f0 = transforme a0 in
	     let s0 = (List.hd !list_txt) in
	     let l = elimImplic s0 f0 in
	     let s1 = List.hd l in 
	     let s2 = List.nth l 1 in
	     let t1 = trans_to_string2 s1.hyp in
	     let t2 = trans_to_string2 s2.hyp in
	     let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	     let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	     let box = List.hd (!branche_active) in
	     erreur#set_text "  ";
	     let box02 =GPack.hbox ~homogeneous:true ~spacing:7 
		 ~packing:((box)#pack ~from:`END) () in
	     let box03 = GPack.vbox ~packing:box02#pack () in
	     let box04 = GPack.vbox ~packing:box02#pack () in
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"(E IMPL)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	     list_txt := List.tl (!list_txt);
	     list_txt := [s1;s2]@(!list_txt);
	     dern_regle := ["E Impl"]@(!dern_regle);
	     for i=1 to 5000000 do()done ;
	     if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     branche_active := List.tl (!branche_active); 
	     branche_active := box03::box04::(!branche_active);
             button0#set_active true;
	     entry0#set_text "  "
	    )
	   else ( 
	     button0#set_active true; 
	     raise FormulePasIndiquee 
	    )
       	  )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
       | FormulePasIndiquee -> (
	   erreur#set_text "entrez une formule" ;
	   button0#set_active true
	  )
      )
				     ) ;
  

(* la fonction concernant le bouton de la rgle elimLeftEt *)  
  button8#connect#toggled  ~callback:(fun _->
    if (button8#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
	   if (a0 <> "  ")then ( 
             if (a1 <> "  ")then (
	       let f0 = transforme a0 in
	       let f1 = transforme2 a1 in
	       let f2 = transforme a2 in
	       let s1 = { hyp = f1;f = f2} in
	       let s2 = List.hd ( elimLeftEt s1 f0) in 
	       let t = trans_to_string2 s2.hyp in
	       let txt =(t^" |- "^(trans_to_string s2.f)) in
	       let box = List.hd (!branche_active) in
	       erreur#set_text "  ";
	       (GMisc.label  ~text:"(E LEFT AND)" ~xalign:0.0 ~yalign:0.0 
		  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box)#pack ~from:`END) () in
	       list_txt := [s2];
	       dern_regle := ["E left and"];
	       for i=1 to 5000000 do()done ;
               button0#set_active true;
	       entry0#set_text "  "
	      )else
	       (
		if (a2<> "  ") then (
		  let f0 = transforme a0 in
		  let f2 = transforme a2 in
		  let s1 = { hyp = [];f = f2} in
		  let s2 = List.hd ( elimLeftEt s1 f0) in 
		  let t = trans_to_string2 s2.hyp in
		  let txt =(t^" |- "^(trans_to_string s2.f)) in
		  let box = List.hd (!branche_active) in
		  erreur#set_text "  ";
		  (GMisc.label  ~text:"(E LEFT AND)" ~xalign:0.0 ~yalign:0.0 
		     ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box)#pack ~from:`END) () in
		  list_txt := [s2];
		  dern_regle := ["E left and"] ;)
		else
		  (
		   entry1#set_editable true;
		   entry2#set_editable true;
		   erreur#set_text "entrez un sequent";
		  );
		for i=1 to 5000000 do()done ;
		button0#set_active true;
		entry0#set_text "  "
	       )
	    )else  ( button0#set_active true ; raise FormulePasIndiquee)
	  )
	 else 
	   (
	    let a0 = entry0#text in
	    if (a0 <> "  ")then (
	      let f0 = transforme a0 in
	      let s1 = (List.hd !list_txt) in
	      let s2 = List.hd ( elimLeftEt s1 f0) in 
	      let t = trans_to_string2 s2.hyp in
	      txt_visite := [ List.hd (!list_txt)]@(!txt_visite);
	      list_txt := List.tl (!list_txt);
	      list_txt := [s2]@(!list_txt) ;
	      dern_regle := ["E left and"]@(!dern_regle);
	      let txt =(t^" |- "^(trans_to_string s2.f)) in
	      let box = List.hd (!branche_active) in
	      erreur#set_text "  ";
	      (GMisc.label  ~text:"(E LEFT AND)" ~xalign:0.0 ~yalign:0.0 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      let separator = GMisc.separator `HORIZONTAL 
		  ~packing: ((box)#pack ~from:`END) () in
	      for i=1 to 5000000 do()done ;
	      if (box <> (List.hd (!branche_visitee))) then  
		branche_visitee := [box]@(!branche_visitee);
	      button0#set_active true;
	      entry0#set_text "  "
	     )
	    else 
	      ( 
		button0#set_active true;
		raise FormulePasIndiquee 
	       )
	   )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
       | FormulePasIndiquee -> (
	   erreur#set_text "entrez une formule" ;
	   button0#set_active true
	  )
      )
				     ) ;

(* la fonction concernant le bouton de la rgle elimRightEt *)
  button9#connect#toggled  ~callback:(fun _->
    if (button9#active)then
      (
       try(  
	 if (entry1#editable & entry2#editable) then (
	   let a0 = entry0#text in
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
	   if (a0 <> "  ")then ( 
             if (a1 <> "  ")then (
	       let f0 = transforme a0 in
	       let f1 = transforme2 a1 in
	       let f2 = transforme a2 in
	       let s1 = { hyp = f1;f = f2} in
	       let s2 = List.hd ( elimRightEt s1 f0) in 
	       let t = trans_to_string2 s2.hyp in
	       let txt =(t^" |- "^(trans_to_string s2.f)) in
	       let box = List.hd (!branche_active) in
	       erreur#set_text "  ";
	       (GMisc.label  ~text:"(E RIGHT AND)" ~xalign:0.0 ~yalign:0.0 
		  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box)#pack ~from:`END) () in
	       list_txt := [s2];
	       dern_regle := ["E right and"];
	       for i=1 to 5000000 do()done ;
               button0#set_active true;
	       entry0#set_text "  "
	      )else
	       (
		if(a2<>"  ") then (
		  let f0 = transforme a0 in
		  let f2 = transforme a2 in
		  let s1 = { hyp = [];f = f2} in
		  let s2 = List.hd ( elimRightEt s1 f0) in 
		  let t = trans_to_string2 s2.hyp in
		  let txt =(t^" |- "^(trans_to_string s2.f)) in
		  let box = List.hd (!branche_active) in
		  erreur#set_text "  ";
		  (GMisc.label  ~text:"(E RIGHT AND)" ~xalign:0.0 ~yalign:0.0 
		     ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box)#pack ~from:`END) () in
		  list_txt := [s2];
		  dern_regle := ["E right and"];)
		else
		  (
		   entry1#set_editable true;
		   entry2#set_editable true;
		   erreur#set_text "entrez un sequent";
		  );
		for i=1 to 5000000 do()done ;
		button0#set_active true;
		entry0#set_text "  "
	       )
	    )else  
	     (
	      button0#set_active true ;
	      raise FormulePasIndiquee
	     )
	  )
	 else 
	   (
	    let a0 = entry0#text in
	    if (a0 <> "  ")then (
	      let f0 = transforme a0 in
	      let s1 = (List.hd !list_txt) in
	      let s2 = List.hd ( elimRightEt s1 f0) in 
	      let t = trans_to_string2 s2.hyp in
	      txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	      list_txt := List.tl (!list_txt);
	      list_txt := [s2]@(!list_txt) ;
	      dern_regle := ["E right and"]@(!dern_regle);
	      let txt =(t^" |- "^(trans_to_string s2.f)) in
	      let box = List.hd (!branche_active) in
	      erreur#set_text;
	      (GMisc.label ~text:"(E RIGHT AND)" ~xalign:0.0 ~yalign:0.0 
		 ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      (GMisc.label ~text:txt ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	      let separator = GMisc.separator `HORIZONTAL 
		  ~packing: ((box)#pack ~from:`END) () in
	      for i=1 to 5000000 do()done ;
	      if (box <> (List.hd (!branche_visitee))) then  
		branche_visitee := [box]@(!branche_visitee);
	      button0#set_active true;
	      entry0#set_text "  "
	     )
	    else ( 
	      button0#set_active true; 
	      raise FormulePasIndiquee
	     )
	   )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
       | FormulePasIndiquee -> (
	   erreur#set_text "entrez une formule" ;
	   button0#set_active true
	  )
      )
				     ) ;
  
(* la fonction concernant le bouton de la rgle elimOu *)
  button10#connect#toggled  ~callback:(fun _->
    if (button10#active)then
      (
       try(
	 if (entry1#editable & entry2#editable) then (
	   let a0 = entry0#text in
	   let a01 = entry01#text in
	   let a1 = entry1#text in
	   let a2 = entry2#text in
	   entry1#set_editable false;
	   entry2#set_editable false;
	   if ((a0 <> "  ")&(a01 <> "  "))then (
             if (a1 <> "  ")then (
	       let f0 = transforme a0 in
	       let f01 = transforme a01 in
	       let f1 = transforme2 a1 in
	       let f2 = transforme a2 in
	       let s0 = { hyp = f1;f = f2} in
	       let l = elimOu s0 f0 f01 in
	       let s1 = List.hd l in
	       let s2 = List.nth l 1 in
	       let s3 = List.nth l 2 in
	       let t1 = trans_to_string2 s1.hyp in
	       let t2 = trans_to_string2 s2.hyp in
	       let t3 = trans_to_string2 s3.hyp in
	       let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	       let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	       let txt3 =(t3^" |- "^(trans_to_string s3.f)) in
	       let box = List.hd (!branche_active) in
	       erreur#set_text "  ";
	       let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		   ~packing:((box)#pack ~from:`END) () in
	       let box03 = GPack.vbox ~packing:box02#pack () in
	       let box04 = GPack.vbox ~packing:box02#pack () in
	       let box05 = GPack.vbox ~packing:box02#pack () in
	       (GMisc.label  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~text:"(E OR)" ~xalign:0.0 ~yalign:0.0 
		  ~packing:((box03)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~packing:((box05)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label ~text:txt3 ~packing:((box05)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       (GMisc.label  ~packing:((box)#pack ~from:`END) 
		  ~line_wrap:true ())#set_use_underline true ;
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box03)#pack ~from:`END) () in
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box04)#pack ~from:`END) () in
	       let separator = GMisc.separator `HORIZONTAL 
		   ~packing: ((box05)#pack ~from:`END) () in
	       list_txt := [s1;s2;s3]@(!list_txt);
	       dern_regle := ["E or"];
	       for i=1 to 5000000 do()done ;
	       if (box <> (List.hd (!branche_visitee))) then  
		 branche_visitee := [box]@(!branche_visitee);
	       branche_active := [box03]@[box04]@[box05];
               button0#set_active true;
	       entry0#set_text "  ";
	       entry01#set_text "  "
	      )else
	       (
		if (a2<>"  ") then (
		  let f0 = transforme a0 in
		  let f01 = transforme a01 in
		  let f2 = transforme a2 in
		  let s0 = { hyp = [];f = f2} in
		  let l = elimOu s0 f0 f01 in
		  let s1 = List.hd l in 
		  let s2 = List.nth l 1 in
		  let s3 = List.nth l 2 in
		  let t1 = trans_to_string2 s1.hyp in
		  let t2 = trans_to_string2 s2.hyp in
		  let t3 = trans_to_string2 s3.hyp in
		  let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
		  let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
		  let txt3 =(t3^" |- "^(trans_to_string s3.f)) in
		  let box = List.hd (!branche_active) in
		  erreur#set_text "  ";
		  let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		      ~packing:((box)#pack ~from:`END) () in
		  let box03 = GPack.vbox ~packing:box02#pack() in
		  let box04 = GPack.vbox ~packing:box02#pack () in
		  let box05 = GPack.vbox ~packing:box02#pack () in
		  (GMisc.label  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~text:"(E OR)" ~xalign:0.0 ~yalign:0.0 
		     ~packing:((box03)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~packing:((box05)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END)  
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END)  
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label ~text:txt3 ~packing:((box05)#pack ~from:`END)  
		     ~line_wrap:true ())#set_use_underline true ;
		  (GMisc.label  ~packing:((box)#pack ~from:`END) 
		     ~line_wrap:true ())#set_use_underline true ;
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box03)#pack ~from:`END) () in
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box04)#pack ~from:`END) () in
		  let separator = GMisc.separator `HORIZONTAL 
		      ~packing: ((box05)#pack ~from:`END) () in
		  list_txt := [s1;s2;s3];
		  dern_regle := ["E or"];
		  if (box <> (List.hd (!branche_visitee))) then 
		    branche_visitee := [box]@(!branche_visitee);
		  branche_active := [box03]@[box04]@[box05];)
		else
		  (
		   entry1#set_editable true;
		   entry2#set_editable true;
		   erreur#set_text "entrez un sequent";
		  );
		for i=1 to 5000000 do()done ;
                button0#set_active true;
		entry0#set_text "  ";
		entry01#set_text "  "
	       )
	    )
	   else
	     ( 
	       button0#set_active true;
	       raise FormulePasIndiquee
	      )
	  )
	 else 
	   (
	    let a0 = entry0#text in
	    let a01 = entry01#text in
	    if (a0 <> "  ")then (
	      let f0 = transforme a0 in
	      let f01 = transforme a01 in
	      let s0 = (List.hd !list_txt)in
	      let l = elimOu s0 f0 f01 in
	      let s1 = List.hd l in 
	      let s2 = List.nth l 1 in
	      let s3 = List.nth l 2 in
	      let t1 = trans_to_string2 s1.hyp in
	      let t2 = trans_to_string2 s2.hyp in
	      let t3 = trans_to_string2 s3.hyp in
	      let txt1 =(t1^" |- "^(trans_to_string s1.f)) in
	      let txt2 =(t2^" |- "^(trans_to_string s2.f)) in
	      let txt3 =(t3^" |- "^(trans_to_string s3.f)) in
	      let box = List.hd (!branche_active) in
	      erreur#set_text "  ";
	      let box02 =GPack.hbox ~homogeneous:true ~spacing:7
		  ~packing:((box)#pack ~from:`END) () in
	      let box03 = GPack.vbox ~packing:box02#pack () in
	      let box04 = GPack.vbox ~packing:box02#pack () in
	      let box05 = GPack.vbox ~packing:box02#pack () in
	      (GMisc.label  ~packing:((box)#pack ~from:`END) 
		 ~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~text:"(E OR)" ~xalign:0.0 ~yalign:0.0 
		~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box05)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt1 ~packing:((box03)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt2 ~packing:((box04)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label ~text:txt3 ~packing:((box05)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     (GMisc.label  ~packing:((box)#pack ~from:`END) 
		~line_wrap:true ())#set_use_underline true ;
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box03)#pack ~from:`END) () in
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box04)#pack ~from:`END) () in
	     let separator = GMisc.separator `HORIZONTAL 
		 ~packing: ((box05)#pack ~from:`END) () in
	     txt_visite := [List.hd (!list_txt)]@(!txt_visite);
	     list_txt := List.tl (!list_txt);
	     list_txt := [s1;s2;s3]@(!list_txt);
	     dern_regle := ["E or"]@(!dern_regle);
	     for i=1 to 5000000 do()done ;
	     if (box <> (List.hd (!branche_visitee))) then  
	       branche_visitee := [box]@(!branche_visitee);
	     branche_active := List.tl (!branche_active); 
	     branche_active := box03::box04::box05::(!branche_active);
             button0#set_active true;
	     entry0#set_text "  ";
	     entry01#set_text "  "
	    )
	   else
	      (
	       button0#set_active true; 
	       raise FormulePasIndiquee 
	      )
       	   )
	)with RegleNonApplicable -> (
	  erreur#set_text "RegleNonApplicable" ;
	  button0#set_active true
	 )
       | Erreur_syntaxe -> (
	   erreur#set_text "Erreur Syntaxique" ;
	   entry1#set_editable true;
	   entry2#set_editable true;
	   button0#set_active true
	  )
       | FormulePasIndiquee -> (
	   erreur#set_text "entrez une formule" ;
	   button0#set_active true
	  )
      )
				      ) ;
  
(* la fonction concernant le bouton de Retour *)  
  b1#connect#clicked ~callback:(fun _->
    if ((!dern_regle)<>[]) then
      (
       let box = List.hd (!branche_visitee) in
       let l1 = box#children in
       if (box = box1) then 
	 (
	  if ((List.length l1)>3) then (
	    box#remove (List.nth l1 0);
	    box#remove (List.nth l1 1);
	    box#remove (List.nth l1 2);
	    if ((List.length l1)=5) then (
	      entry1#set_editable true;
	      entry2#set_editable true;
	     )
	   )
	 )
       else (
	 box#remove (List.nth l1 0);
	 box#remove (List.nth l1 1);
	 box#remove (List.nth l1 2);
	) ;
       let d = (List.hd (!dern_regle)) in
       (
	match d  with     
	| "hyp" -> 
	    ( 
              if ((!branche_active)<> []) then
		(
		 if ((List.hd (!branche_active))<>(List.hd (!branche_visitee)))
		 then
		   branche_active := 
		     [List.hd (!branche_visitee)]@(!branche_active);
		)
	      else branche_active := 
		[List.hd (!branche_visitee)]@(!branche_active);
	      if ((List.length l1)= 6)then 
		branche_visitee := List.tl (!branche_visitee);
	      if ((!txt_visite) <> []) then (
		list_txt := [(List.hd (!txt_visite))]@(!list_txt);
		txt_visite := List.tl (!txt_visite);))
	| "I Impl" ->   
	    ( 
	      if ((List.length l1)= 6)then 
		branche_visitee := List.tl (!branche_visitee);
	      list_txt := List.tl (!list_txt);
	      if ((!txt_visite) <> []) then (
		list_txt := [(List.hd (!txt_visite))]@(!list_txt);
		txt_visite := List.tl (!txt_visite);
	       )
	     )
	| "I and" ->   
	    (
	     list_txt := List.tl (List.tl (!list_txt));
	     branche_active := List.tl (List.tl (!branche_active));
	     if ((!branche_active)<> []) then
	       (
		if ((List.hd (!branche_active))<>(List.hd (!branche_visitee)))
		then
		  branche_active := 
		    [List.hd (!branche_visitee)]@(!branche_active);
	       )
	     else branche_active := 
	       [List.hd (!branche_visitee)]@(!branche_active);
	     if ((List.length l1)= 6)then
	       branche_visitee := List.tl (!branche_visitee);
	     if ((!txt_visite) <> []) then (
	       list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	       txt_visite := List.tl (!txt_visite);))
	| "I left or" ->
	    ( 
	      if ((List.length l1)= 6)then 
		branche_visitee := List.tl (!branche_visitee);
	      list_txt := List.tl (!list_txt);
	      if ((!txt_visite) <> []) then (
		list_txt := [(List.hd (!txt_visite))]@(!list_txt);
		txt_visite := List.tl (!txt_visite);))
	| "I right or" ->    
	    ( 
	      if ((List.length l1)= 6)then 
		branche_visitee := List.tl (!branche_visitee);
              list_txt := List.tl (!list_txt);
	      if ((!txt_visite) <> []) then (
		list_txt := [(List.hd (!txt_visite))]@(!list_txt);
		txt_visite := List.tl (!txt_visite)))
      | "E false" ->
	  ( 
	    if ((List.length l1)= 6)then
	      branche_visitee := List.tl (!branche_visitee);
	    list_txt := List.tl (!list_txt);
	    if ((!txt_visite) <> []) then (
	      list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	      txt_visite := List.tl (!txt_visite)))
      | "E Impl" -> 
	  (
	   list_txt := List.tl (List.tl (!list_txt));
	   branche_active := List.tl (List.tl (!branche_active));
	   if ((!branche_active)<> []) then
	     (
	      if ((List.hd (!branche_active))<>(List.hd (!branche_visitee)))
	      then
		branche_active :=
		  [List.hd (!branche_visitee)]@(!branche_active);
	     )
	   else
	     branche_active := [List.hd (!branche_visitee)]@(!branche_active);
	   if ((List.length l1)= 6)then 
	     branche_visitee := List.tl (!branche_visitee);
	   if ((!txt_visite) <> []) then (
	     list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	     txt_visite := List.tl (!txt_visite)))
      | "E left and" -> 
	  (
	   if ((List.length l1)= 6)then 
	     branche_visitee := List.tl (!branche_visitee);
	   list_txt := List.tl (!list_txt);
	   if ((!txt_visite) <> []) then (
	     list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	     txt_visite := List.tl (!txt_visite)))
      | "E right and" ->
	  (
	   if ((List.length l1)= 6)then 
	     branche_visitee := List.tl (!branche_visitee);
	   list_txt := List.tl (!list_txt);
	   if ((!txt_visite) <> []) then (
	     list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	     txt_visite := List.tl (!txt_visite)))
      | "E or" -> 
	  ( 
	    list_txt := List.tl (List.tl (List.tl (!list_txt)));
	    branche_active := List.tl (List.tl (List.tl (!branche_active)));
	    if ((!branche_active)<> []) then
	      (
	       if ((List.hd (!branche_active))<>(List.hd (!branche_visitee)))
	       then
		 branche_active := 
		   [List.hd (!branche_visitee)]@(!branche_active);
	      )
	    else branche_active := 
	      [List.hd (!branche_visitee)]@(!branche_active);
	    if ((List.length l1)= 6)then 
	      branche_visitee := List.tl (!branche_visitee);
	    if ((!txt_visite) <> []) then (
	      list_txt := [(List.hd (!txt_visite))]@(!list_txt);
	      txt_visite := List.tl (!txt_visite)
	     )
	   )
       );  
       dern_regle := List.tl (!dern_regle);
       if ((d <> "I Impl") || 
       ((List.hd (!dern_regle)) <> "E Impl")||
       (erreur#text<> "COUPURE")) then  erreur#set_text "   ";	
      )
			       );
  
(* la fonction concernant le bouton de Recommencer *)  
  b0#connect#clicked ~callback:(fun _->
    let l = box1#children in
    for i=0 to ((List.length (l))-3) do 
      (
       let n = (List.nth l i) in
       box1#remove (n);
      )done ;
    for i=0 to ((List.length (!branche_visitee))-1) do 
      (
       let n = (List.nth (!branche_visitee) i) in
       if (n <> box1) then vider n;
      )done ;
    for i=0 to ((List.length (!branche_active))-1) do 
      (
       let n = (List.nth (!branche_active) i) in
       if (n <> box1) then vider n;
      )done ;
    branche_active := [box1];
    branche_visitee:= [box1];
    list_txt := [] ;
    txt_visite := [];
    dern_regle := [];
    entry1#set_editable true;
    entry2#set_editable true;
    entry0#set_text "  ";
    entry01#set_text "  ";
    entry1#set_text "  ";
    entry2#set_text "  ";
    erreur#set_text "  ";
    button0#set_active true
			       );
  window#show ();
  Main.main ()
    
let _ = main ()
    
    
